-- Zakuun Assist --

local ENCOUNTERID_FelLordZakuun = 1391
local SPELLID_SeedofDestruction = 181508
local DEBUFF_APPLY_DELAY_MAX = 0.5

local frame = CreateFrame("Frame", "ZakuunAssistFrame", UIParent)
local frameEvents = {};

--/run ZakuunAssist_Debug("debug")

--/run ZakuunAssist_Debug("newspell",77761);ZakuunAssist_Debug("startcombat")
--/run ZakuunAssist_Debug("endcombat")
local isDebug = false
local function debugLog(...)
  if isDebug then
    print("ZakuunAssist", ...)
  end
end
function ZakuunAssist_Debug(msg, i)
  if msg == "debug" then
    isDebug = not isDebug
    print("ZakuunAssist: debug", isDebug)
  elseif msg == "startcombat" then
    frameEvents["ENCOUNTER_START"](frame, ENCOUNTERID_FelLordZakuun)
  elseif msg == "endcombat" then
    frameEvents["ENCOUNTER_END"](frame)
  elseif msg == "newspell" then
    --77761: Stampeding Roar (bear form)
    if type(i) == "number" then
      SPELLID_SeedofDestruction = i
      debugLog("newspell =", SPELLID_SeedofDestruction, GetSpellInfo(i))
    end
  end
end

function frameEvents:ENCOUNTER_START(encounterID)
  if encounterID == ENCOUNTERID_FelLordZakuun then
    debugLog("encounter started!")
    self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
  end
end

function frameEvents:ENCOUNTER_END()
  debugLog("encounter ended.")
  self:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
end

function frameEvents:COMBAT_LOG_EVENT_UNFILTERED(...)
  local _, event, _, _, _, _, _, destGUID = select(1, ...)
  if event == "SPELL_AURA_APPLIED" then
    local spellId = select(12, ...) --from prefix SPELL
    if spellId == SPELLID_SeedofDestruction then
      if (destGUID:sub(1, 7)) == "Player-" then
        debugLog("new debuff on player!")
        self:addDebuffedPlayer(destGUID)
      end
    end
  elseif event == "SPELL_AURA_REMOVED" then
    local spellId = select(12, ...) --from prefix SPELL
    if spellId == SPELLID_SeedofDestruction then
      if (destGUID:sub(1, 7)) == "Player-" then
        debugLog("debuff removed from player.")
        self:removeDebuffedPlayer(destGUID)
      end
    end
  end
end

frame:SetScript("OnEvent", function(self, event, ...)
  frameEvents[event](self, ...)
end);

do --init
  frame.debuffCounter = 0
  frame.debuffedPlayers = {};
  frame.myGUID = UnitGUID("player")
  
  frame.doTheMath = function(self)
    debugLog("marking players now")
    local nextTargetIcon = 1
    local guid
    local unitId
    for i = 1, MAX_PARTY_MEMBERS do
      unitId = "raid"..i
      guid = UnitGUID(unitId) or ""
      if self.debuffedPlayers[guid] then
        debugLog("  debuffed player found with unitId", unitId)
        self.debuffedPlayers[guid] = unitId --cache unitId
        SetRaidTarget(unitId, nextTargetIcon)
        
        if guid == self.myGUID then
          --/say
          SendChatMessage("{rt"..nextTargetIcon.."}", "SAY")
        end
        
        nextTargetIcon = nextTargetIcon + 1
      end
    end
  end
  frame.addDebuffedPlayer = function(self, guid)
    debugLog("addDebuffedPlayer(guid =", guid, ")")
    self.debuffedPlayers[guid] = true
    if self.debuffCounter == 0 then
      self:startTimer()
    end
    self.debuffCounter = self.debuffCounter + 1
  end
  frame.removeDebuffedPlayer = function(self, guid)
    debugLog("removeDebuffedPlayer(guid =", guid, ")")
    local unitId = self.debuffedPlayers[guid]
    if unitId then
      debugLog("  unitId found, removing...")
      SetRaidTarget(unitId, 0)
      
      self.debuffedPlayers[guid] = nil
      self.debuffCounter = self.debuffCounter - 1
      if self.debuffCounter == 0 then
        debugLog("  no more players")
        self.debuffedPlayers = {};
      end
    end
  end
  
  frame.tslU = -1
  frame.interval = DEBUFF_APPLY_DELAY_MAX
  
  frame.NoOp = function() end
  frame.RealOnUpdate = function(self, elapsed)
    self.tslU = self.tslU + elapsed
    if self.tslU >= self.interval then
      debugLog("timer finished")
      self.tslU = -1
      self.OnUpdate = self.NoOp
      
      self:doTheMath()
    end
  end
  frame.OnUpdate = frame.NoOp
  frame:SetScript("OnUpdate", function(self, elapsed)
    self.OnUpdate(self, elapsed)
  end);
  
  frame.startTimer = function(self)
    debugLog("startTimer()")
    self.tslU = 0
    self.OnUpdate = self.RealOnUpdate
  end
  
  frame:RegisterEvent("ENCOUNTER_START")
  frame:RegisterEvent("ENCOUNTER_END")
end
